﻿package {
	import flash.display.Shape;
	import flash.display.Sprite;
	import flash.display.MovieClip;
	import flash.events.Event;
	import flash.text.TextField;
	import funnel.*;
	import funnel.i2c.*;

	/**
	 * ボタンが押されている間LEDをサイン波でドライブする。
	 * このサンプルではデジタルとアナログの入出力に加えてI2Cデバイスを
	 * 利用する方法を紹介する。
	 * Drive a LED with sin wave while the button is pressed.
	 * This example demonstrates the use of 2 I2C devices along with digital
	 * and analog I/O.
	 *
	 * <p>準備<ol>
	 * <li>D12にスイッチを接続する（プルアップが必要）</li>
	 * <li>D11にLEDを接続する（電流制限のための抵抗器が必要）</li>
	 * <li>A0にセンサを接続する（例：ボリューム）</li>
	 * <li>HMC6352（デジタルコンパス）、LIS3LV02DQ（加速度センサ）をI2Cバスに接続する</li>
	 * </ol></p>
	 *
	 * <p>Preparation<ol>
	 * <li>Connect a button to D12 (should be pulled-up)</li>
	 * <li>Connect a LED to D11 (current-limiting resistor is needed)</li>
	 * <li>Connect a sensor to A0 (e.g. a potentiometer)</li>
	 * <li>Connect a HMC6352 (compass), LIS3LV02DQ (accelerometer), or other I2C device to the I2C bus</li>
	 * </ol></p>
	 */
	public class FioTestWithI2C extends Sprite {
		// To change number of analog channels, modify this constant
		// 表示するアナログチャンネル数を変更するにはこの定数を変更する
		private const NUM_CHANNELS:int = 1;

		private var fio:Fio;
		private var scope:SimpleScope;
		private var scopes:Array;
		private var acclScopes:Array;
		private var accl:LIS302DL;
		private var compass:HMC6352;
		private var clockHands:Array;

		public function FioTestWithI2C() {
			var config:Configuration = Fio.FIRMATA;
			config.setDigitalPinMode(12, IN);
			config.setDigitalPinMode(13, OUT);
			
			// set unused analog pins to digital outputs to save processor time
			config.setDigitalPinMode(15, OUT);	// set A1 as digital output
			config.setDigitalPinMode(16, OUT);	// set A2 as digital output
			config.setDigitalPinMode(17, OUT);	// set A3 as digital output		
			config.setDigitalPinMode(20, OUT);	// set A6 as digital output		
			config.setDigitalPinMode(21, OUT);	// set A7 as digital output		

			fio = new Fio([1], config);
			fio.addEventListener(FunnelEvent.READY, trace);
			fio.addEventListener(FunnelErrorEvent.REBOOT_ERROR, trace);
			fio.addEventListener(FunnelErrorEvent.CONFIGURATION_ERROR, trace);
			fio.addEventListener(FunnelErrorEvent.ERROR, trace);
			
			// You can set the second parameter to "false" to disable Read Continuous mode
			// Read Continuous mode is recommended because it significantly
			// improves the Flash Players performance since the Arduino handles
			// most of the work in reading the I2C device
			accl = new LIS302DL(fio.ioModule(1));		// default mode = Read Continuous
			//accl = new LIS3LV02DQ(fio.ioModule(1), false);	// this will disable Read Continuous
			                                        // you will need to call the update method
													// to request data from the device
			accl.addEventListener(Event.CHANGE, onAcclUpdate);
			
			acclScopes = new Array(3);
			for (var i:int = 0; i < 3; i++) {
				acclScopes[i] = new SimpleScope(130, 10 + (60 * i), 200, true);
				addChild(acclScopes[i]);
			}
			
			compass =  new HMC6352(fio.ioModule(1));
			compass.addEventListener(Event.CHANGE, onCompassUpdate);
			
			clockHands = new Array();
			var offset = { x:60, y:185 };
			for (i = 0; i < 3; i++)
			{
				var clockHand:Shape = new Shape();
				clockHand.x = offset.x + i*60;
				clockHand.y = offset.y;
				clockHand.graphics.clear();
				clockHand.graphics.beginFill(0x000000);
				clockHand.graphics.moveTo(0, -25);
				clockHand.graphics.lineTo(5, 5);
				clockHand.graphics.lineTo(0, 0);
				clockHand.graphics.lineTo(-5, 5);
				clockHand.graphics.lineTo(0, -25);
				clockHand.graphics.endFill();
				clockHand.graphics.lineStyle(1, 0x000000);
				clockHand.graphics.drawCircle(0, 0, 26);
				clockHand.rotation = 0;
				this.addChild(clockHand);
				clockHands.push(clockHand);
			}

			scopes = new Array(NUM_CHANNELS);
			for (i = 0; i < NUM_CHANNELS; i++) {
				scopes[i] = new SimpleScope(10, 10 + (60 * i), 200);
				addChild(scopes[i]);
			}

			var circle:Shape = new Shape();
			circle.graphics.beginFill(0x000000);
			circle.graphics.drawCircle(0, 0, 25);
			circle.graphics.endFill();
			circle.alpha = 0.2;
			circle.x = 110;
			circle.y = 270;
			this.addChild(circle);

			var button:Pin = fio.ioModule(1).digitalPin(12);
			var onBoardLED:Pin = fio.ioModule(1).digitalPin(13);

			button.addEventListener(PinEvent.FALLING_EDGE, function(e:Event):void {
				onBoardLED.value = 1.0;
				circle.alpha = 1.0;
			});

			button.addEventListener(PinEvent.RISING_EDGE, function(e:Event):void {
				onBoardLED.value = 0.0;
				circle.alpha = 0.2;
			});

			addEventListener(Event.ENTER_FRAME, loop);
		}

		private function loop(event:Event):void {
			for (var i:int = 0; i < NUM_CHANNELS; i++) {
				scopes[i].update(fio.ioModule(1).analogPin(i));
			}
			
			// only need to call this method if Read Continuous mode is not enabled
			//accl.update(); // request new x, y, z values from accelerometer			
		}
		
		private function onAcclUpdate(event:Event):void {
			acclScopes[0].updateValue(accl.x);
			acclScopes[1].updateValue(accl.y);
			acclScopes[2].updateValue(accl.z);
			textContainer.xVal.text = String(accl.x);
			textContainer.yVal.text = String(accl.y);
			textContainer.zVal.text = String(accl.z);
			
			// get x-axis and y-axis degrees 
			var roll:Number = Math.atan2(accl.z, accl.y)/ Math.PI * 180.0; //Math.asin(accl.x) / Math.PI * 180.0;
			var pitch:Number = Math.atan2(accl.z, accl.x) / Math.PI * 180.0;
			clockHands[0].rotation = roll;
			clockHands[1].rotation = pitch;
			textContainer.rollVal.text = String(roll);
			textContainer.pitchVal.text = String(pitch);
		}
		
		private function onCompassUpdate(event:Event):void {
			// get z-axis degree 
			var yaw:Number = compass.heading;
			clockHands[2].rotation = yaw;
			textContainer.heading.text = String(yaw);
		}
	}
}
